#!/bin/bash
########################################################################
# build_pandora.sh
#
# Build pandora and package for UPS.
#
# Adapted from build.sh.template provided by ssibuildshims v2_07_00.
########################################################################

########################################################################
# Non-customize-able boilerplate: skip to "Customize-able functions."
########################################################################

####################################
# Useful variables.
prog=${BASH_SOURCE[0]##*/}

##################
# Local (dumb) ssi_die() function for use until we source ssi_functions.
ssi_die() {
  printf "FATAL ERROR: ${*}\n" 1>&2; exit 1
}
##################

##################
# Basic usage function for use by ensure_buildshims until we have enough
# information to provide something more specific. Do NOT customize!
build_script_usage() {
    cat <<EOF
USAGE: ${prog} <ssi-msg-config-options> [--] <product-dir> <qualifier-args> <maketar>
       ${prog} -h|--help [<product-dir>]

  Build ${package} and package for UPS.

OPTIONS

  -h|--help

    Help. Supply <product-dir> for specific, detailed information on
    invoking ${prog}.

  <ssi-msg-config-options>

    Supply <product-dir> for more information.

ARGUMENTS

  <product-dir>

    The top-level UPS directory in which the source code distribution
    was unpacked. When specified with -h|--help, this enables us to
    provide more specific help about invoking ${prog}.

  <qualifier-args>
  <maketar>

    Supply <product-dir> for more information.

EOF
}
##################

##################
# Make sure we have UPS and ssibuildshims set up.
ensure_ssibuildshims() {
  if ! (( $# )) || [ "${@: -1}" = "-h" ] || [ "${@: -1}" = "--help" ]; then
    build_script_usage
    exit 1
  fi
  # Find <product-dir> from our arguments:
  local arg
  for arg in "${@}"; do
    [[ "$arg" ==  -* ]] && continue;
    product_dir="${arg}"; break
  done
  if [ -z "${product_dir}" ]; then
    # Bad usage.
    ssi_die "Bad <product-dir> ${product_dir}\n$(build_script_usage)"
    exit 2
  elif ! { [ -d "${product_dir}/.upsfiles" ] && [ -w "${product_dir}" ]; }; then
    # Something wrong with <product-dir>.
    ssi_die "<product-dir> not a valid, writable unified UPS directory: " \
      "${product_dir}\n$(build_script_usage)"
  fi
  # Promising...
  local abs_product_dir="$(cd "${product_dir}" 2>/dev/null 2>&1 && pwd -P)"
  (( $? )) && ssi_die "unable to cd to <product-dir>: ${product_dir}\n" \
    "$(build_script_usage)"
  # Find our own bootstraps and yank 'em.
  local ssibuildshims_topdir="${product_dir}/ssibuildshims/${ssibuildshims_version}"
  [ -f "${ssibuildshims_topdir}/bin/ssi_functions" ] && \
    source "${ssibuildshims_topdir}/bin/ssi_functions" >/dev/null 2>&1 && \
    ensure_ups "${product_dir}" && \
    setup ssibuildshims ${ssibuildshims_version} -z "${product_dir}" || \
    ssi_die "unable to find ssibuildshims ${ssibuildshims_version} in
${product_dir} and set it up with UPS."
  # Absolute is safer.
  absolute_path -- "${product_dir}" product_dir
}
##################

########################################################################
# Customize-able functions.
########################################################################

##################
# Ascertain whether we built and installed correctly. Add more /
# different calls to verify_in_filesystem as appropriate. Arguments
# (e.g. -q) are passed through to verify_in_filesystem.
is_built() { # *C*
  ${trace} verify_in_filesystem -d "${@}" "${pkgdir}/lib"{64,}
}
##################

##################
# If a distribution archive is wanted and the installation is successful
# according to is_built(), then make it. Any arguments (e.g. -q) are
# passed through both to is_built() and make_basic_distribution().
#
# If make_basic_distribution() finds that the product is declared
# correctly and attempts to make the distribution, then it will cause us
# to exit with the status code of that operation. If not, it returns an
# error code.
#
# See make_basic_distribution -h for useful options to add to
# mbd_opts(). *C*
maybe_make_distribution() {
  if [ "${maketar}" = "tar" ]; then
    # Customize mbd_opts. *C*
    local mbd_opts=()
    ${trace} is_built "$@" && \
      ${trace} make_basic_distribution "${@}" "${mbd_opts[@]}"
  fi
}
##################

########################################################################
# Package-specific functions.
########################################################################

# ...

########################################################################
# Main.
########################################################################

####################################
# Useful variables. *C*
trace=maybe_xtrace # Could set to xtrace instead for unconditional.
(( ssi_trace = 1 )) # Trace certain commands by default.
####################################

####################################
# Package name and version information. *C*
package=pandora
origpkgver=v03_15_00
pkgver=${origpkgver} # Append letters for local patches.
ssibuildshims_version=v2_07_00
pkgdotver=${origpkgver//_/.}; pkgdotver=${pkgdotver#v}
pndver=`echo ${origpkgver} | sed -e 's/_/-/g'`
pkgtarfile=${package}-${pndver}.tar
PandoraPFA_ver=`echo ${origpkgver} | sed -e 's/_/-/g'`
PandoraSDK_ver=v03-04-00
PandoraMonitoring_ver=v03-05-00
LCContent_ver=v03-01-05
####################################

##################
# Other useful variables. *C*
pkgtarname=${package}-${pndver}
pkgtarfile=${pkgtarname}.tar
##################

####################################
# Handle options and arguments.

# Set up ssibuildshims and UPS / product_dir as early as possible.
ensure_ssibuildshims "${@}"

##################
# Now parse options and arguments, setting basequal, extraqual, maketar,
# etc.
#
# Use verify_quals() to parse command line options, verify that the user
# has provided qualifiers supported by this build script, and set
# expected variables such as basequal, extraqual, cqual, etc.
#
# Customize the extra usage text as appropriate. Note that DEBUG_PATCHES
# and VERBOSE_PATCHES are honored by prepare_source() and
# patch_source(). DEBUG_(CONFIG|BUILD|TEST|INSTALL) should be accounted
# for (or not) by the relevant command in this script.
#
# See verify_quals -h for options and examples.
verify_quals \
  --cqual={e19,c7} \
  --pqual={,py2} \
  --btype={debug,prof} \
  --parse --usage-file=- -- "$@" <<EOF || exit # *C*
ENVIRONMENT

  DEBUG_(CONFIG|BUILD|TEST|INSTALL)

    Verbose output from the corresponding step.

  DEBUG_PATCHES
  VERBOSE_PATCHES

    Command tracing and increased verbosity for source unpacking and patching
    operations. DEBUG_PATCHES will cause exit after patches have been applied.
EOF
##################

##################
# Define globals: pkgdir, sourcedir, blddir, fullqual, etc. See
# define_basics -h for steering options.
${trace} define_basics TRACK_PROGRESS IN_SOURCE || exit # *C*
##################

####################################
# Declare ourselves privately, and then set ourselves up to allow setup
# of dependencies. *C*
#
# Comment or delete if not required:

${trace} fake_declare_product "${ssi_args[@]}" -x -- || exit
if [ -z ${ROOTSYS} ]
then
   echo "ERROR: failed to setup root"
   exit 1
else
   echo "ROOTSYS: ${ROOTSYS}"
   echo
fi

####################################

####################################
# Build-only dependencies. *C*
#
# e.g.
#   ${trace} setup_cmake --latest || exit
# AND/OR
#   ${trace} setup_ninja --optional && (( want_ninja = 1 ))

${trace} setup_cmake --latest || exit

####################################

# Print active products.
echo
ups active
echo

# Possible short-circuit if we're already built and declared.
${trace} maybe_make_distribution --silent

ssi_info "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${pkgflvr})"

# Ensure directories: pkgdir, sourcedir, blddir.
${trace} init_pkg_dirs || exit

####################################
# Prepare source for building. *C*

${trace} cd "${sourcedir}" \
  || ssi_die "unable to cd to source directory ${sourcedir}"

# Source tarballs and patches. See prepare_source -h / patch_source -h
# for options.
# pandora has multiple tarballs....
mkdir -p ${PANDORA_FQ_DIR}/${package}-${pndver} || ssi_die "failed to create ${PANDORA_FQ_DIR}/${package}-${pndver}"
${trace} cd ${PANDORA_FQ_DIR}/${package}-${pndver}
for rep in PandoraPFA PandoraSDK PandoraMonitoring LCContent
do
  ${trace} tar xf ${tardir}/${rep}-*.tar  || ssi_die "failed to unwind ${tardir}/${rep}-*.tar"
done

# Bail early if DEBUG_PATCHES is set.
[ -n "${DEBUG_PATCHES}" ] && exit 1
####################################

####################################
# Prepare to build. *C*
#
# Set configure / compile flags, environment variables, etc.

if [[ "${basequal}" == e[79] ]] || [[ "${basequal}" == e[79]:nu ]]
then
  cc=gcc
  cxx=g++
  cxxflg="-std=c++14"
elif [[ "${basequal}" == e1[045] ]] || [[ "${basequal}" == e1[045]:nu ]]
then
  cc=gcc
  cxx=g++
  cxxflg="-std=c++14"
elif [[ "${basequal}" == e1[79] ]] 
then
  cc=gcc
  cxx=g++
  cxxflg="-std=c++17 -Wno-implicit-fallthrough"
elif [[ "${basequal}" == e1[79]:py2 ]] 
then
  cc=gcc
  cxx=g++
  cxxflg="-std=c++17 -Wno-implicit-fallthrough"
elif [[ "${basequal}" == c[27] ]]
then
  cc=clang
  cxx=clang++
  cxxflg="-std=c++17 -D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1"
elif [[ "${basequal}" == c[27]:py2 ]]
then
  cc=clang
  cxx=clang++
  cxxflg="-std=c++17 -D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1"
else
   echo "ERROR: unrecognized qualifier ${basequal}"
   usage
   exit 1
fi

# ...

##################
# Configure parallelism. *C*
ncores=$(ncores)

# For cmake --build:
#
# [[ -n "${CMAKE_BUILD_PARALLEL_LEVEL}" ]] || \
#   export CMAKE_BUILD_PARALLEL_LEVEL=${ncores}
#
# For ctest:
#
# [[ -n "${CTEST_PARALLEL_LEVEL}" ]] || \
#   export CTEST_PARALLEL_LEVEL=${ncores}
##################
####################################

####################################
# Configure, build, and install.
#
# Use ${blddir}/${pkgtarname} for in-source builds, ${blddir} otherwise.

${trace} cd "${blddir}" || \
  ssi_die "unable to cd to build directory ${blddir}"

##################
# Configure. *C*

# e.g.
#   ${trace} ${sourcedir}/${pkgtarname}/configure \
#     --prefix="${pkgdir}" ... || ssi_die "configure failed"
# OR
#   ${trace} cmake <opts> ${DEBUG_CONFIG:+--verbose} \
#     "${sourcedir}/${pkgtarname}" || \
#     ssi_die "configure failed"
##################

##################
# Build. *C*

# e.g.
#   ${trace} cmake --build "${blddir}" ${DEBUG_BUILD:+--verbose} || \
#     ssi_die "build failed"

# we have to pass the compiler info to the sub-builds
export CC=${cc} 
export CXX=${cxx}
export FC=gfortran
for rep in PandoraSDK PandoraMonitoring LCContent
do
  mkdir ${blddir}/${rep} || ssi_die "Failed to create directory ${blddir}/${rep}"
  cd ${blddir}/${rep}
  rep_ver=${rep}_ver
  set -x
  cmake ${extra_command}  \
       -DCMAKE_MODULE_PATH=${PANDORA_FQ_DIR}/${package}-${pndver}/PandoraPFA-${PandoraPFA_ver}/cmakemodules \
       -DCMAKE_INSTALL_PREFIX=${PANDORA_FQ_DIR} \
       -DROOT_DIR=${ROOTSYS}/etc/cmake \
       -DCMAKE_CXX_FLAGS="${cxxflg}" \
       -DPANDORA_LC_CONTENT=ON \
       ${PANDORA_FQ_DIR}/${package}-${pndver}/${rep}-${!rep_ver} || ssi_die "cmake for ${rep} failed"
  make -j${ncores} install
  set +x
done

##################

##################
# Test. *C*

# e.g.
# (( WANT_TESTS )) && \
#   { ${trace} ctest --output-on-failure || ssi_die "tests failed"; }
##################

##################
# Install. *C*

# e.g.
#   ${trace} cmake --install "${blddir}" || ssi_die "install failed"
##################

ssi_info "finished building ${package} ${pkgver}"

####################################
# Pre-declaration processing. *C*
#
# Move things, install from contrib, clean build products from in-source
# builds, etc.

ssi_info "post-processing for relocatability"

fixfile=/tmp/pandorafix$$$$.sh
cmakeConfig=`ls ${pkgdir}/*.cmake ${pkgdir}/lib/cmake/*.cmake`
for file in ${cmakeConfig}
do
  echo "mv ${file} ${file}.bak" >> ${fixfile}
  echo "cat ${file}.bak | sed 's%${pkgdir}%\$ENV{PANDORA_FQ_DIR}%g' | sed 's%${ROOTSYS}%\$ENV{ROOTSYS}%g' > ${file}" >> ${fixfile}
done

echo "-------------"
cat ${fixfile}
echo "-------------"
chmod +x ${fixfile}
${fixfile} || ssi_die "ERROR: problem running ${fixfile}."

rm -f ${fixfile}
rm -f ${pkgdir}/*.cmake.bak
rm -f ${pkgdir}/lib/cmake/*.cmake.bak

# fix_config_script is not adding $ENV to cmake config files
#fix_config_script \
#  =PANDORA_FQ_DIR \
#  =ROOTSYS \
#  -- \
#  "${pkgdir}"/{PandoraMonitoringConfig.cmake,PandoraMonitoringConfigVersion.cmake,PandoraSDKConfig.cmake,PandoraSDKConfigVersion.cmake} \
#  "${pkgdir}"/lib/cmake/{PandoraMonitoringLibDeps.cmake,PandoraSDKLibDeps.cmake} || exit

####################################

##################
# Declare and set up the installed package. *C*
#
# Add -c for current declaration if required.
${trace} declare_product -x -- || exit
##################

####################################
# Post-declaration processing. *C*
#
# Operations requiring the *final* declaration and setup (as opposed to
# the pre-build fake declaration and setup). This should be rare.
####################################

#################
# Verify installation and make distribution archive. *C*
#
# To customize verification, see is_built(), since it is also used by
# maybe_make_distribution(). See the documentation for those functions
# (above) for customization opportunities.

# Add a message to ssi_build_failed if appropriate.
${trace} is_built && ${trace} ssi_build_completed && \
  ssi_info "${package} is installed at ${pkgdir}" && \
  ${trace} maybe_make_distribution || \
  ssi_build_failed # *C*
########################################################################
# Nothing beyond this point.
########################################################################
